/*
 * Copyright 2026 Element Creations Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.textualEvent {
    font-size: var(--cpd-font-size-body-sm);
    line-height: normal;

    overflow-y: hidden;
    color: var(--cpd-color-text-secondary);

    a {
        cursor: pointer;
        color: var(--cpd-color-text-action-accent);
    }
}

.textualEvent[data-event-layout="irc"] {
    padding: 1px 0; /* add a 1px padding top and bottom because our larger emoji font otherwise gets cropped by anti-zalgo */
    display: inline-block;
    line-height: 1.125rem; /* $font-18px equivalent of the legacy --irc-line-height */
}
