/*
 * Copyright 2025 New Vector Ltd.
 *
 * SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
 * Please see LICENSE files in the repository root for full details.
 */

.button {
    position: relative;
    width: 32px;
    height: 32px;
    min-width: 32px;
    min-height: 32px;
    border-radius: 32px;
    background-color: var(--cpd-color-bg-subtle-primary);
    padding: var(--cpd-space-1-5x);
    box-sizing: border-box;
}

.button svg {
    width: 20px;
    height: 20px;
}

.button[disabled] svg {
    opacity: 0.5;
}
