/*
Copyright 2026 Element Creations Ltd.

SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-Element-Commercial
Please see LICENSE files in the repository root for full details.
*/

import type { ModuleFactory } from "@element-hq/element-web-module-api";
import WidgetLifecycleModule from "./WidgetLifecycleModule";

export default WidgetLifecycleModule satisfies ModuleFactory;
