/*
Copyright 2025 New Vector Ltd.

SPDX-License-Identifier: AGPL-3.0-only OR GPL-3.0-only OR LicenseRef-Element-Commercial
Please see LICENSE files in the repository root for full details.
*/

.docs-story {
    background: var(--cpd-color-bg-canvas-default);
}

[class*="blurhash"] > canvas {
    animation: none !important;
    opacity: 1 !important;
}
