/*
Copyright 2025 New Vector Ltd.

SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-Element-Commercial
Please see LICENSE files in the repository root for full details.
*/

/// <reference types="vite-plugin-svgr/client" />
/// <reference types="@arcmantle/vite-plugin-import-css-sheet/client" />
